require 'socket'

module Balotelli
  module Core
    module IRC
      def configure(host, port, nick, pass = nil)
        @host = host
        @port = port
        @nick = nick
        @pass = pass

        @socket = nil
        @mutex = Mutex.new
      end

      def connect(&block)
        @socket = TCPSocket.new(@host, @port)

        sputs("PASS #{@pass}") if @pass
        sputs("NICK #{@nick}")
        sputs("USER #{@nick} 0 * :#{@nick}")

        instance_variable_set(:@on_connect, block) if block_given?
      end

      def sgets
        str = @socket.gets
        str.chomp! unless str.nil?

        puts '<< ' + str.inspect

        str
      end

      def sputs(str)
        @mutex.synchronize do
          puts '>> ' + str.inspect

          @socket.puts(str)
        end
      end

      def pong(message)
        sputs("PONG #{message}")
      end

      def join(channel, password = nil)
        sputs("JOIN #{channel} #{password}")
      end

      def privmsg(channel, message)
        sputs("PRIVMSG #{channel} :#{message}")
      end

      def names(channel)
        sputs("NAMES #{channel}")
      end
    end
  end
end
