require 'logger'
require 'fileutils'
require 'set'
module Balotelli
  module Core
    class IrcLogger < Module
      LOG_FORMAT = proc do |_severity, datetime, _progname, msg|
        "#{datetime.utc}: #{msg}\n"
      end

      module Methods
        def sgets
          str = orig_sgets
          to_log = "<<#{str.inspect}\n"
          @log_file.info(to_log)
          if str =~ /\A:?\S+ \S+ (#\S+) .*/
            @logger.channel_log(Regexp.last_match(1).downcase, to_log)
          end
          str
        end

        def sputs(str)
          orig_sputs(str)
          to_log = ">>#{str.inspect}\n"
          @log_file.info(to_log)
          if str =~ /\A:?\S+ (#\S+) .*/
            @logger.channel_log(Regexp.last_match(1).downcase, to_log)
          end
          str
        end
      end

      attr_reader :log_file, :dir, :channels, :channels_logs

      def initialize(dir = '', log_name = 'logs.log', channels = [])
        @dir = dir
        check_dir
        @log_file = new_logger(log_name)
        @channels = Set.new channels
        @channels_logs = @channels.each_with_object({}) do |c, o|
          o[c] = new_logger(channel_log_file(c))
        end
      end

      def included(des)
        super

        des.instance_variable_set(:@log_file, log_file)
        des.instance_variable_set(:@logger, self)

        define_log_method

        class << des
          alias_method :orig_sputs, :sputs
          private :orig_sputs

          alias_method :orig_sgets, :sgets
          private :orig_sgets
        end
        des.extend(Methods)
      end

      def channel_log(channel, to_log)
        if (logger = @channels_logs[channel])
          logger.info to_log
        end
      end

      def define_log_method
        define_method :log_file do |str|
          class_variable_get(:@log_file)
        end

        define_method :log do |str|
          log_string = "LOG: #{str.inspect}\n"
          log_file.info log_string
          puts log_string
        end
      end

      private

      def channel_log_file(channel)
        "#{channel.tr('#', '')}_logs.log"
      end

      def new_logger(file_name)
        Logger.new(File.join(dir, file_name), 0, 1.0 / 0).tap do |l|
          l.formatter = LOG_FORMAT
        end
      end

      def check_dir
        FileUtils.mkdir_p(dir) unless File.file?(dir)
      end
    end
  end
end
